package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.*;
import org.apache.shiro.authc.AuthenticationException;
import java.util.List;

/**
 * @Description : 用户账号服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 16:21
 * @Modify :
 **/
public interface ISysUserService {

    /**
     * 用户登录
     * @param loginName     登录名
     * @param password      登录密码
     * @param languageCode  界面语言
     * @return
     */
    SessionUser queryUserLogin(String loginName, String password, String languageCode) throws AuthenticationException;

    /**
     * 修改权限状态
     * @param id        修改权限状态
     * @param status    状态
     * @param user      当前会话用户
     * @return
     */
    void updateSysUserStatus(Long id, int status, SessionUser user);

    /**
     * 批量修改用户状态
     * @param ids
     * @param status
     * @param user
     */
    void updateBatchSysUserStatus(Long[] ids, int status, SessionUser user);

    /**
     * 修改用户信息
     * @param sysUser
     */
    void updateSysUser(SysUser sysUser) throws Exception;

    /**
     * 修改用户密码
     * @param userId     需要修改的用户ID
     * @param password      旧密码
     * @param newPwd        新密码
     */
    void updateSysUserPassword(Long userId,String password,String newPwd)  throws Exception;

    /**
     * 刷新用户登录信息
     * @param userId
     */
    void refreshUserLoginInformation(Long userId);

    /**
     * 新增用户信息
     * @param sysUser 新增用户
     * @return
     */
    SysUser insertSysUser(SysUser sysUser)throws Exception;

    /**
     * 刷新用户角色关系
     * @param sysUser
     * @param roleIds
     * @param userName
     */
    void refreshSysRefUserRole(SysUser sysUser, Long[] roleIds , String userName);

    /**
     * 根据Key 删除数据
     * @param id
     * @return
     */
    void deleteSysUserById(Long id);

    /**
     * 批量删除用户 逻辑删除
     * @param ids   需要删除的用户ID
     * @param user  当前会话用户
     */
    void deleteBatchSysUserById(Long[] ids, SessionUser user);

    /**
     * 查询所有数据
     * @return
     */
    List<SysUser> list();

    /**
     * 复杂查询
     * @param user
     * @param pager
     * @return
     */
    ListPager<SysUser> querySysUserByPager(SysUser user, Pager pager);

    /**
     * 根据id 数据
     * @param id
     * @return
     */
    SysUser getSysUserById(Long id);

    /**
     * 获取用户详细信息
     * @param id
     * @return
     */
    SysUserInfo getSysUserInfoById(Long id);

    /**
     * 根据登录名称 获取用户信息
     * @param loginName
     * @return
     */
    SysUser getSysUserByLoginName(String loginName);

    /**
     * 根据手机号查询账号
     * @param email
     * @return
     */
    SysUser getSysUserByEmail(String email);

    /**
     *  根据手机号查询账号
     * @param phone
     * @return
     */
    SysUser getSysUserByPhone(String phone);

    /**
     * 查出用户角色关系
     * @param userId
     * @return
     */
    List<SysRefUserRole> findSysRefUserRoleByUserId(Long userId);

    /**
     * 判断用户名是否存在
     * @param userLoginName
     * @param id
     * @return
     */
    boolean checkSysUserLoginName(String userLoginName, long id);

    /**
     * 判断邮箱是否存在
     * @param email
     * @param id
     * @return
     */
    boolean checkSysUserEmail(String email,long id);

    /**
     * 判断手机是否存在
     * @param phone
     * @param id
     * @return
     */
    boolean checkSysUserPhone(String phone,long id);

    /**
     * 用户唯一教研
     * @param user
     */
    void checkSysUserOnly(SysUser user);
}
