package cn.estsh.i3plus.core.apiservice.controller;

import cn.estsh.impp.framework.boot.util.ResultBean;
import com.netflix.appinfo.ApplicationInfoManager;
import com.netflix.appinfo.EurekaInstanceConfig;
import com.netflix.discovery.EurekaClient;
import com.netflix.discovery.EurekaClientConfig;
import com.netflix.discovery.shared.Applications;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description :
 * @Reference :
 * @Author :  Alwaysfrin
 * @CreateDate : 2018-11-19 19:56
 * @Modify:
 **/
@RestController
@Api(description="微服功能demo")
@RequestMapping("/demo-eureka-service")
@ConditionalOnExpression("'${eureka.client.enabled}' == 'true'")
public class DemoEurekaController {

    private static final Logger LOGGER = LoggerFactory.getLogger(DemoEurekaController.class);

    @Autowired
    private EurekaClient eurekaClient;

    @Autowired
    private ApplicationInfoManager applicationInfoManager;

    /**
     * 停止微服
     * @return
     */
    @GetMapping(value="/stop")
    @ApiOperation(value="停止微服",notes = "停止当前微服")
    public ResultBean stopServer() {
        LOGGER.info("请求停止微服。。。");
        eurekaClient.shutdown();
        return new ResultBean(true);
    }

    /**
     * 停止微服
     * @return
     */
    @GetMapping(value="/show")
    @ApiOperation(value="显示微服",notes = "显示微服")
    public ResultBean show() {
        LOGGER.info("【加载微服配置】");
        LOGGER.info("applicationInfoManager:{}",applicationInfoManager);
        LOGGER.info("getEurekaInstanceConfig:{}",applicationInfoManager.getEurekaInstanceConfig());

        EurekaInstanceConfig config = applicationInfoManager.getEurekaInstanceConfig();
        LOGGER.info(config.getAppname());

        Applications app = eurekaClient.getApplications();
        LOGGER.info("spring dc : "+eurekaClient + ",app：" + app);

        EurekaClientConfig eurekaClientConfig = eurekaClient.getEurekaClientConfig();
        LOGGER.info("eurekaClientConfig:{}",eurekaClientConfig);

        return new ResultBean(true);
    }
}
