package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreTreeService;
import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ModelEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.model.platform.CommonTreeModel;
import cn.estsh.i3plus.pojo.platform.bean.SysDepartment;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysOrganize;
import cn.estsh.i3plus.pojo.platform.bean.SysPosition;
import cn.estsh.i3plus.pojo.platform.repository.SysDepartmentRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysMenuRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysOrganizeRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysPositionRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : Core 树功能实现
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 11:17
 * @Modify :
 **/
@Service
public class CoreTreeService implements ICoreTreeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(CoreTreeService.class);

    @Autowired
    private SysDepartmentRepository departmentRDao;

    @Autowired
    private SysPositionRepository positionRDao;

    @Autowired
    private SysOrganizeRepository organizeRDao;

    @Autowired
    private SysMenuRepository menuRDao;

    @Override
    public List<SysPosition> findSysPositionTreeByParentId(long parentId) {
        List<SysPosition> result = positionRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            for (SysPosition position : result) {
                findSysPositionChildrenTreePack(position,0);
            }
        }

        return result;
    }

    @Override
    public SysPosition getSysPositionTreeById(long id) {
        // 查找数据
        SysPosition position = positionRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(position,"不存在的岗位信息");
        // 递归查询子集
        findSysPositionChildrenTreePack(position, 0);

        return position;
    }

    @Override
    public void findSysPositionChildrenTreePack(SysPosition parent, int step) {
        ++step;
        List<SysPosition> childList = positionRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {
            //说明有子集
            for (SysPosition child : childList) {
                findSysPositionChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public void doValidatorSysPositionParentId(SysPosition position) {
        findSysPositionChildrenTreePack(position,0);
        validatorSysPositionParentId(position.getChildList(),position.getParentId());
    }

    @Override
    public List<SysDepartment> findSysDepartmentTreeByParentId(long parentId) {
        List<SysDepartment> result = departmentRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            for (SysDepartment department : result) {
                findSysDepartmentChildrenTreePack(department,0);
            }
        }

        return result;
    }

    @Override
    public SysDepartment getSysDepartmentTreeById(long id) {
        // 查找数据
        SysDepartment department = departmentRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(department,"不存在的部门信息");
        // 递归查询子集
        findSysDepartmentChildrenTreePack(department, 0);

        return department;
    }

    @Override
    public void findSysDepartmentChildrenTreePack(SysDepartment parent, int step) {
        ++step;
        List<SysDepartment> childList = departmentRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {
            //说明有子集
            for (SysDepartment child : childList) {
                findSysDepartmentChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public void doValidatorSysDepartmentParentId(SysDepartment department) {
        findSysDepartmentChildrenTreePack(department,0);
        validatorSysDepartmentParentId(department.getChildList(),department.getParentId());
    }

    @Override
    public List<SysOrganize> findSysOrganizeTreeByParentId(long parentId) {
        List<SysOrganize> result = organizeRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            for (SysOrganize organize : result) {
                findSysOrganizeChildrenTreePack(organize,0);
            }
        }

        return result;
    }

    @Override
    public SysOrganize getSysOrganizeTreeById(long id) {
        // 查找数据
        SysOrganize organize = organizeRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(organize,"不存在的组织信息");
        // 递归查询子集
        findSysOrganizeChildrenTreePack(organize, 0);

        return organize;
    }

    @Override
    public void findSysOrganizeChildrenTreePack(SysOrganize parent, int step) {
        ++step;
        List<SysOrganize> childList = organizeRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {  //说明有子集
            for (SysOrganize child : childList) {
                findSysOrganizeChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public void doValidatorSysOrganizeParentId(SysOrganize organize) {
        findSysOrganizeChildrenTreePack(organize,0);
        validatorSysOrganizeParentId(organize.getChildList(),organize.getParentId());
    }

    @Override
    public List<SysMenu> findSysMenuTreeByParentId(long parentId) {
        List<SysMenu> result = menuRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            for (SysMenu menu : result) {
                findSysMenuChildrenTreePack(menu,0);
            }
        }

        return result;
    }

    @Override
    public SysMenu getSysMenuTreeById(long id) {
        // 查找数据
        SysMenu menu = menuRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(menu,"不存在的组织信息");
        // 递归查询子集
        findSysMenuChildrenTreePack(menu, 0);

        return menu;
    }

    @Override
    public void findSysMenuChildrenTreePack(SysMenu parent, int step) {
        ++step;
        List<SysMenu> childList = menuRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {  //说明有子集
            for (SysMenu child : childList) {
                findSysMenuChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public void doValidatorSysMenuParentId(SysMenu menu) {
        findSysMenuChildrenTreePack(menu,0);
        validatorSysMenuParentId(menu.getChildList(),menu.getParentId());
    }

    private void validatorSysPositionParentId(List<SysPosition> list, Long parentId){
        if(list != null && list.size() > 0){
            for (SysPosition position : list) {
                // 判断子集的ID 是否是 父节点ID
                if(position.getId().equals(parentId) || position.getId().equals(position.getParentId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("上级岗位不能为子岗位")
                            .setErrorSolution("请重新操作")
                            .build();
                }

                validatorSysPositionParentId(position.getChildList(),parentId);
            }
        }
    }

    private void validatorSysDepartmentParentId(List<SysDepartment> list, Long parentId){
        if(list != null && list.size() > 0){
            for (SysDepartment department : list) {
                // 判断子集的ID 是否是 父节点ID
                if(department.getId().equals(parentId) || department.getId().equals(department.getParentId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("上级部门不能为子部门")
                            .setErrorSolution("请重新操作")
                            .build();
                }

                validatorSysDepartmentParentId(department.getChildList(),parentId);
            }
        }
    }

    private void validatorSysOrganizeParentId(List<SysOrganize> list, Long parentId){
        if(list != null && list.size() > 0){
            for (SysOrganize organize : list) {
                // 判断子集的ID 是否是 父节点ID
                if(organize.getId().equals(parentId) || organize.getId().equals(organize.getParentId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("上级部门不能为子部门")
                            .setErrorSolution("请重新操作")
                            .build();
                }

                validatorSysOrganizeParentId(organize.getChildList(),parentId);
            }
        }
    }

    private void validatorSysMenuParentId(List<SysMenu> list, Long parentId){
        if(list != null && list.size() > 0){
            for (SysMenu menu : list) {
                // 判断子集的ID 是否是 父节点ID
                if(menu.getId().equals(parentId) || menu.getId().equals(menu.getParentId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                            .setErrorDetail("上级部门不能为子部门")
                            .setErrorSolution("请重新操作")
                            .build();
                }

                validatorSysMenuParentId(menu.getChildList(),parentId);
            }
        }
    }

    @Override
    public List<CommonTreeModel> findOrganizeDepartment(long parentId) {
        List<CommonTreeModel> result = new ArrayList<>();

        StringBuffer hql = new StringBuffer();
        HqlPack.getNumEqualPack(parentId,"parentId",hql);
        HqlPack.getOrderByPack(new Object[]{2}, new String[]{"organizeSort"}, hql);
        List<SysOrganize> list = organizeRDao.findByHqlWhere(hql.toString());

        // 循环设置子集
        if(list != null && list.size() > 0){
            for (SysOrganize organize : list) {
                CommonTreeModel treeModel = new CommonTreeModel();
                treeModel.setBean(organize);
                treeModel.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_ORGANIZE.getValue());

                findOrganizeTreePack(treeModel,0);

                result.add(treeModel);
            }
        }


        return result;
    }

    @Override
    public CommonTreeModel getOrganizeById(long id) {
        // 查找数据
        SysOrganize organize = organizeRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(organize,"不存在的组织信息");

        CommonTreeModel treeModel = new CommonTreeModel();
        treeModel.setBean(organize);
        treeModel.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_ORGANIZE.getValue());

        // 递归查询子集
        findOrganizeTreePack(treeModel, 0);
        return treeModel;
    }

    @Override
    public void findOrganizeTreePack(CommonTreeModel parent, int step) {
        if(null != parent){
            BaseBean bean = parent.getBean();
            List<CommonTreeModel> childList = new ArrayList<>();
            CommonTreeModel treeModel = null;
            if(bean instanceof SysOrganize){
                SysOrganize organize = (SysOrganize) bean;

                StringBuffer hql = new StringBuffer();
                HqlPack.getNumEqualPack(organize.getId(),"parentId",hql);
                HqlPack.getOrderByPack(new Object[]{2}, new String[]{"organizeSort"}, hql);
                List<SysOrganize> organizeList = organizeRDao.findByHqlWhere(hql.toString());

                if(organizeList != null && organizeList.size() > 0){
                    for (SysOrganize org : organizeList) {
                        treeModel = new CommonTreeModel();
                        treeModel.setBean(org);
                        treeModel.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_ORGANIZE.getValue());

                        findOrganizeTreePack(treeModel,step);

                        childList.add(treeModel);
                    }
                }

                StringBuffer depHQL = new StringBuffer();
                HqlPack.getNumEqualPack(organize.getId(),"organizeId",depHQL);
                HqlPack.getOrderByPack(new Object[]{2}, new String[]{"departmentSort"}, depHQL);
                List<SysDepartment> departmentList = departmentRDao.findByHqlWhere(depHQL.toString());

                if(departmentList != null && departmentList.size() > 0){
                    for (SysDepartment department : departmentList) {
                        if(department.getParentId() != null && department.getParentId() < 0){
                            treeModel = new CommonTreeModel();
                            treeModel.setBean(department);
                            treeModel.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_DEPARTMENT.getValue());

                            findOrganizeTreePack(treeModel,step);

                            childList.add(treeModel);
                        }
                    }
                }
                LOGGER.info("【{}】包含子集：{},步长：{}", parent.getBean().getClass().getName(), parent.getChildList().size(), step);
            }else if(bean instanceof SysDepartment){
                SysDepartment department = (SysDepartment) bean;

                StringBuffer depHQL = new StringBuffer();
                HqlPack.getNumEqualPack(department.getId(),"parentId",depHQL);
                HqlPack.getOrderByPack(new Object[]{2}, new String[]{"departmentSort"}, depHQL);
                List<SysDepartment> departmentList = departmentRDao.findByHqlWhere(depHQL.toString());

                if(departmentList != null && departmentList.size() > 0){
                    for (SysDepartment dep : departmentList) {
                        treeModel = new CommonTreeModel();
                        treeModel.setBean(dep);
                        treeModel.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_DEPARTMENT.getValue());

                        findOrganizeTreePack(treeModel,step);

                        childList.add(treeModel);
                    }
                }
            }
            parent.setChildList(childList);
        }
    }
}
